/*
    Copyright 2008 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "blowfish.h"

#ifdef WIN32
    #include <windows.h>
#else
    #define stristr strcasestr
#endif

typedef unsigned char   u8;



#define VER         "0.1"



void darkspace_decrypt(u8 *string);
u8 *decode(u8 *data, int *size, u8 *table);
void wait_exit(int ret);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE    *fd     = NULL;
    int     found   = 0;
    u8      buff[4096],
            *fname,
            *p;

    setbuf(stdout, NULL);

    fputs("\n"
        "DarkSpace passwords decrypter "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stderr);

    if(argc < 2) {
        fprintf(stderr, "\n"
            "Usage: %s [config.ini/password]\n"
            "\n", argv[0]);
        wait_exit(1);
    }
    fname = argv[1];

    if(!strcmp(fname, "-")) {
        fd = stdout;
    } else {
        //fprintf(stderr, "- open file %s\n", fname);
        fd = fopen(fname, "rb");
    }
    if(!fd) {
        p = strchr(fname, '=');
        if(p) {
            p++;
        } else {
            p = fname;
        }
        darkspace_decrypt(p);
        fprintf(stderr, "\n- decrypted string:\n\n");
        printf("%s\n", p);
        found++;
    } else {
        fprintf(stderr, "- loaded file %s\n\n", fname);
        while(fgets(buff, sizeof(buff), fd)) {
            for(p = buff; *p && (*p != '\r') && (*p != '\n'); p++);
            *p = 0;
            p = strchr(buff, '=');
            if(!p) continue;
            *p++ = 0;
            if(!*p) continue;
            if(stristr(buff, "previousPW") || stristr(buff, "pwd") || stristr(buff, "pass")) {
                darkspace_decrypt(p);
                printf("  %-20s %s\n", buff, p);
                found++;
            }
        }
        if(fd != stdout) fclose(fd);
    }

    if(!found) fprintf(stderr, "- no passwords found\n");
    wait_exit(0);
    return(0);
}



void darkspace_decrypt(u8 *string) {
    blf_ctx ctx;
    int     len;
    u8      *key,
            *pwd;

    pwd = strstr(string, "[ECM]");
    if(!pwd || (strlen(pwd) <= 0x25)) return;
    pwd += 5;
    key = pwd + strlen(pwd) - 32;   // the key is the md5 hash of the GetComputerName followed by GetUserName like: "mycomputerAdministrator"

    pwd++;  // uudecode uses the first byte, my function doesn't
    len = key - pwd;
    pwd = decode(pwd, &len, "`!\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_");
    blf_key(&ctx, key, strlen(key));
    blf_dec(&ctx, (void *)pwd, len);

    memmove(string, pwd, len + 1);
}



u8 *decode(u8 *data, int *size, u8 *table) {
    int     len,
            xlen,
            a,
            b,
            c;
    u8  *buff,
            *limit,
            *p;
    static u8 base[128];

    if(!table) return(NULL);
    for(p = table; *p; p++) {
        if(*p >= 0x80) return(NULL);;
        base[*p] = p - table;
    }

    if(!size || (*size < 0)) {      // use size -1 for auto size!
        len = strlen(data);
    } else {
        len = *size;
    }
    xlen = ((len >> 2) * 3) + 1;    // NULL included in output for text
    buff = malloc(xlen);
    if(!buff) return(NULL);

    p = buff;
    limit = data + len;
    for(;;) {
#define B64NEXT do {            /* auto chars cleaning */   \
                    c = *data;                              \
                    if(data == limit) {                     \
                        c = 0;                              \
                        break;                              \
                    }                                       \
                    data++;                                 \
                } while(c && ((c <= ' ') || (c > 0x7f)));   \
                if(!c) break;

        B64NEXT;
        a = base[c];

        B64NEXT;
        b = base[c];
        *p++ = (a << 2)        | (b >> 4);

        B64NEXT;
        a = base[c];
        *p++ = ((b & 15) << 4) | (a >> 2);

        B64NEXT;
        *p++ = ((a & 3) << 6)  | base[c];

#undef B64NEXT
    }
    *p = 0;

    len = p - buff;
    if(size) *size = len;
    if((len + 1) != xlen) {             // save memory
        buff = realloc(buff, len + 1);  // NULL included!
        if(!buff) return(NULL);
    }
    return(buff);
}



void wait_exit(int ret) {
#ifdef WIN32
    if(GetWindowLong(GetForegroundWindow(), GWL_WNDPROC)) {
        fprintf(stderr, "\n  press RETURN to exit\n");
        fgetc(stdin);
    }
#endif
    exit(ret);
}



void std_err(void) {
    perror("\nError");
    wait_exit(1);
}


